@@ -13,7 +13,7 @@ if sys.platform == "win32":
 from argyll_cgats import cal_to_fake_profile
 from argyll_names import video_encodings
 from config import (get_data_path, get_verified_path, getcfg, geticon, hascfg,
-					setcfg)
+					profile_ext, setcfg)
 from log import safe_print
 from meta import name as appname, version
 from options import debug
@@ -139,6 +139,10 @@ class LUT3DFrame(BaseFrame):
 										   self.lut3d_trc_black_output_offset_ctrl_handler)
 		self.lut3d_trc_black_output_offset_intctrl.Bind(wx.EVT_TEXT,
 											  self.lut3d_trc_black_output_offset_ctrl_handler)
+		self.lut3d_hdr_peak_luminance_ctrl.Bind(wx.EVT_SPINCTRL,
+											self.lut3d_hdr_peak_luminance_handler)
+		self.lut3d_hdr_peak_luminance_ctrl.Bind(wx.EVT_KILL_FOCUS,
+											self.lut3d_hdr_peak_luminance_handler)
 		self.encoding_input_ctrl.Bind(wx.EVT_CHOICE,
 											self.lut3d_encoding_input_ctrl_handler)
 		self.encoding_output_ctrl.Bind(wx.EVT_CHOICE,
@@ -219,6 +223,10 @@ class LUT3DFrame(BaseFrame):
 		setcfg("3dlut.output.profile.apply_cal",
 			   int(self.lut3d_apply_cal_cb.GetValue()))
 
+	def lut3d_hdr_peak_luminance_handler(self, event):
+		setcfg("3dlut.hdr_peak_luminance",
+			   self.lut3d_hdr_peak_luminance_ctrl.GetValue())
+
 	def lut3d_trc_black_output_offset_ctrl_handler(self, event):
 		if event.GetId() == self.lut3d_trc_black_output_offset_intctrl.GetId():
 			self.lut3d_trc_black_output_offset_ctrl.SetValue(
@@ -257,13 +265,21 @@ class LUT3DFrame(BaseFrame):
 			self.lut3d_set_option("3dlut.trc_gamma", 2.4)
 			self.lut3d_set_option("3dlut.trc_gamma_type", "B")
 			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
+			setcfg("3dlut.trc", "bt1886")
 			self.lut3d_update_trc_controls()
 		elif self.lut3d_trc_ctrl.GetSelection() == 0:
 			# Pure power gamma 2.2
 			self.lut3d_set_option("3dlut.trc_gamma", 2.2)
 			self.lut3d_set_option("3dlut.trc_gamma_type", "b")
 			self.lut3d_set_option("3dlut.trc_output_offset", 1.0)
+			setcfg("3dlut.trc", "gamma2.2")
 			self.lut3d_update_trc_controls()
+		elif self.lut3d_trc_ctrl.GetSelection() == 2:  # SMPTE 2084, hard clip
+			setcfg("3dlut.trc", "smpte2084.hardclip")
+		elif self.lut3d_trc_ctrl.GetSelection() == 3:  # SMPTE 2084, roll-off clip
+			setcfg("3dlut.trc", "smpte2084.rolloffclip")
+		else:
+			setcfg("3dlut.trc", "customgamma")
 		self.lut3d_show_trc_controls()
 		self.Thaw()
 
@@ -513,6 +529,8 @@ class LUT3DFrame(BaseFrame):
 						ext = "txt"
 					elif ext == "madVR":
 						ext = "3dlut"
+					elif ext == "icc":
+						ext = profile_ext[1:]
 					defaultFile = os.path.splitext(defaultFile or
 												   os.path.basename(config.defaults.get("last_3dlut_path")))[0] + "." + ext
 					dlg = wx.FileDialog(self, 
@@ -663,7 +681,10 @@ class LUT3DFrame(BaseFrame):
 		output_encoding = getcfg("3dlut.encoding.output")
 		if (getcfg("3dlut.apply_trc") or
 			not hasattr(self, "lut3d_trc_apply_none_ctrl")):
-			trc_gamma = getcfg("3dlut.trc_gamma")
+			if getcfg("3dlut.trc").startswith("smpte2084"):
+				trc_gamma = getcfg("3dlut.trc")
+			else:
+				trc_gamma = getcfg("3dlut.trc_gamma")
 		else:
 			trc_gamma = None
 		trc_gamma_type = getcfg("3dlut.trc_gamma_type")
@@ -675,6 +696,7 @@ class LUT3DFrame(BaseFrame):
 		output_bits = getcfg("3dlut.bitdepth.output")
 		apply_black_offset = getcfg("3dlut.apply_black_offset")
 		use_b2a = getcfg("3dlut.gamap.use_b2a")
+		white_cdm2 = getcfg("3dlut.hdr_peak_luminance")
 		try:
 			self.worker.create_3dlut(profile_in, path, profile_abst,
 									 profile_out, apply_cal=apply_cal,
@@ -687,7 +709,7 @@ class LUT3DFrame(BaseFrame):
 									 trc_gamma_type=trc_gamma_type,
 									 trc_output_offset=outoffset,
 									 apply_black_offset=apply_black_offset,
-									 use_b2a=use_b2a)
+									 use_b2a=use_b2a, white_cdm2=white_cdm2)
 		except Exception, exception:
 			return exception
 		return True
@@ -909,8 +931,27 @@ class LUT3DFrame(BaseFrame):
 							self.update_layout()
 							self.Thaw()
 					else:
-						self.Freeze()
 						if which == "input":
+							if (not hasattr(self, which + "_profile") or
+								getcfg("3dlut.%s.profile" % which) !=
+								profile.fileName):
+								# Get profile blackpoint so we can check if it makes
+								# sense to show TRC type and output offset controls
+								try:
+									odata = self.worker.xicclu(profile, (0, 0, 0),
+															   pcs="x")
+								except Exception, exception:
+									show_result_dialog(exception, self)
+									self.set_profile_ctrl_path(which)
+									return
+								else:
+									if len(odata) != 1 or len(odata[0]) != 3:
+										show_result_dialog("Blackpoint is invalid: %s"
+														   % odata, self)
+										self.set_profile_ctrl_path(which)
+										return
+									self.XYZbpin = odata[0]
+							self.Freeze()
 							self.lut3d_trc_label.Show()
 							self.lut3d_trc_apply_none_ctrl.Show()
 							self.lut3d_trc_apply_black_offset_ctrl.Show()
@@ -928,32 +969,11 @@ class LUT3DFrame(BaseFrame):
 							self.lut3d_apply_cal_cb.Show()
 							self.lut3d_show_encoding_controls()
 							self.lut3d_update_encoding_controls()
-							if (not hasattr(self, which + "_profile") or
-								getcfg("3dlut.%s.profile" % which) !=
-								profile.fileName):
-								# Get profile blackpoint so we can check if it makes
-								# sense to show TRC type and output offset controls
-								try:
-									odata = self.worker.xicclu(profile, (0, 0, 0),
-															   pcs="x")
-								except Exception, exception:
-									wx.CallAfter(show_result_dialog, exception,
-												 self)
-								else:
-									if len(odata) != 1 or len(odata[0]) != 3:
-										show_result_dialog("Blackpoint is invalid: %s"
-														   % odata, self)
-									self.XYZbpin = odata[0]
 							# Update controls related to output profile
 							setattr(self, "input_profile", profile)
 							if not self.set_profile("output", silent=silent):
 								self.update_linking_controls()
 						elif which == "output":
-							enable_apply_cal = (isinstance(profile.tags.get("vcgt"),
-														   ICCP.VideoCardGammaType))
-							self.lut3d_apply_cal_cb.SetValue(enable_apply_cal and
-													   bool(getcfg("3dlut.output.profile.apply_cal")))
-							self.lut3d_apply_cal_cb.Enable(enable_apply_cal)
 							if (not hasattr(self, which + "_profile") or
 								getcfg("3dlut.%s.profile" % which) !=
 								profile.fileName):
@@ -963,13 +983,22 @@ class LUT3DFrame(BaseFrame):
 									odata = self.worker.xicclu(profile, (0, 0, 0),
 															   pcs="x")
 								except Exception, exception:
-									wx.CallAfter(show_result_dialog, exception,
-												 self)
+									show_result_dialog(exception, self)
+									self.set_profile_ctrl_path(which)
+									return
 								else:
 									if len(odata) != 1 or len(odata[0]) != 3:
 										show_result_dialog("Blackpoint is invalid: %s"
 														   % odata, self)
+										self.set_profile_ctrl_path(which)
+										return
 									self.XYZbpout = odata[0]
+							self.Freeze()
+							enable_apply_cal = (isinstance(profile.tags.get("vcgt"),
+														   ICCP.VideoCardGammaType))
+							self.lut3d_apply_cal_cb.SetValue(enable_apply_cal and
+													   bool(getcfg("3dlut.output.profile.apply_cal")))
+							self.lut3d_apply_cal_cb.Enable(enable_apply_cal)
 							self.gamut_mapping_inverse_a2b.Enable()
 							allow_b2a_gamap = ("B2A0" in profile.tags and
 											   isinstance(profile.tags.B2A0,
@@ -998,12 +1027,10 @@ class LUT3DFrame(BaseFrame):
 												 (getcfg("3dlut.format") != "madVR" or
 												  self.output_profile_ctrl.IsShown()))
 					return profile
-			getattr(self, "%s_profile_ctrl" %
-						  which).SetPath(getcfg("3dlut.%s.profile" % which))
+			self.set_profile_ctrl_path(which)
 		else:
 			if which == "input":
-				getattr(self, "%s_profile_ctrl" %
-							  which).SetPath(getcfg("3dlut.%s.profile" % which))
+				self.set_profile_ctrl_path(which)
 				self.lut3d_update_encoding_controls()
 			else:
 				if not silent:
@@ -1012,6 +1039,10 @@ class LUT3DFrame(BaseFrame):
 					if which == "output":
 						self.lut3d_apply_cal_cb.Disable()
 						self.lut3d_create_btn.Disable()
+
+	def set_profile_ctrl_path(self, which):
+		getattr(self, "%s_profile_ctrl" %
+					  which).SetPath(getcfg("3dlut.%s.profile" % which))
 	
 	def setup_language(self):
 		BaseFrame.setup_language(self)
@@ -1033,7 +1064,8 @@ class LUT3DFrame(BaseFrame):
 	def lut3d_setup_language(self):
 		# Shared with main window
 		items = []
-		for item in ("Gamma 2.2", "trc.rec1886", "custom"):
+		for item in ("Gamma 2.2", "trc.rec1886", "trc.smpte2084.hardclip",
+					 "trc.smpte2084.rolloffclip", "custom"):
 			items.append(lang.getstr(item))
 		self.lut3d_trc_ctrl.SetItems(items)
 		
@@ -1170,7 +1202,13 @@ class LUT3DFrame(BaseFrame):
 			self.Parent.lut3d_update_shared_controls()
 
 	def lut3d_update_trc_control(self):
-		if (getcfg("3dlut.trc_gamma_type") == "B" and
+		if getcfg("3dlut.trc").startswith("smpte2084"):  # SMPTE 2084
+			if getcfg("3dlut.trc") == "smpte2084.hardclip":
+				sel = 2
+			else:
+				sel = 3
+			self.lut3d_trc_ctrl.SetSelection(sel)
+		elif (getcfg("3dlut.trc_gamma_type") == "B" and
 			getcfg("3dlut.trc_output_offset") == 0 and
 			getcfg("3dlut.trc_gamma") == 2.4):
 			self.lut3d_trc_ctrl.SetSelection(1)  # BT.1886
@@ -1179,7 +1217,7 @@ class LUT3DFrame(BaseFrame):
 			getcfg("3dlut.trc_gamma") == 2.2):
 			self.lut3d_trc_ctrl.SetSelection(0)  # Pure power gamma 2.2
 		else:
-			self.lut3d_trc_ctrl.SetSelection(2)  # Custom
+			self.lut3d_trc_ctrl.SetSelection(4)  # Custom
 
 	def lut3d_update_trc_controls(self):
 		self.lut3d_update_trc_control()
@@ -1188,6 +1226,7 @@ class LUT3DFrame(BaseFrame):
 		outoffset = int(getcfg("3dlut.trc_output_offset") * 100)
 		self.lut3d_trc_black_output_offset_ctrl.SetValue(outoffset)
 		self.lut3d_trc_black_output_offset_intctrl.SetValue(outoffset)
+		self.lut3d_hdr_peak_luminance_ctrl.SetValue(getcfg("3dlut.hdr_peak_luminance"))
 
 	def update_linking_controls(self):
 		self.gamut_mapping_inverse_a2b.SetValue(
@@ -1211,18 +1250,18 @@ class LUT3DFrame(BaseFrame):
 				# Rec. 709 TRC
 				setcfg("3dlut.apply_trc",
 					   int(tf[0][1] in (-240, -709) or
-						   tf[0][0].startswith("Gamma")))
+						   (tf[0][0].startswith("Gamma") and tf[1] >= .95)))
 				# Use only BT.1886 black output offset
 				setcfg("3dlut.apply_black_offset",
 					   int(tf[0][1] not in (-240, -709) and
-						   not tf[0][0].startswith("Gamma") and
+						   (not tf[0][0].startswith("Gamma") or tf[1] < .95) and
 						   self.XYZbpin != self.XYZbpout))
 			self.lut3d_trc_apply_black_offset_ctrl.Enable(
 				tf[0][1] not in (-240, -709) and
 				self.XYZbpin != self.XYZbpout)
 			# Set gamma to profile gamma if single gamma
 			# profile
-			if tf[0][0].startswith("Gamma"):
+			if tf[0][0].startswith("Gamma") and tf[1] >= .95:
 				if not getcfg("3dlut.trc_gamma.backup", False):
 					# Backup current gamma
 					setcfg("3dlut.trc_gamma.backup",
@@ -1268,9 +1307,10 @@ class LUT3DFrame(BaseFrame):
 		if hasattr(self, "lut3d_trc_apply_ctrl"):
 			self.lut3d_trc_apply_ctrl.Show(show)
 		self.lut3d_trc_ctrl.Show(show)
-		show = show and (self.lut3d_trc_ctrl.GetSelection() == 2 or  # Custom
-						 (isinstance(self, LUT3DFrame) or
-						  getcfg("show_advanced_options")))
+		show = (show and (getcfg("3dlut.trc") == "customgamma" or
+						  (isinstance(self, LUT3DFrame) or
+						   getcfg("show_advanced_options"))) and
+				not getcfg("3dlut.trc").startswith("smpte2084"))
 		self.lut3d_trc_gamma_label.Show(show)
 		self.lut3d_trc_gamma_ctrl.Show(show)
 		show = show and ((hasattr(self, "lut3d_create_cb") and
@@ -1280,6 +1320,10 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_trc_black_output_offset_ctrl.Show(show)
 		self.lut3d_trc_black_output_offset_intctrl.Show(show)
 		self.lut3d_trc_black_output_offset_intctrl_label.Show(show)
+		show = getcfg("3dlut.trc").startswith("smpte2084")  # SMPTE 2084
+		self.lut3d_hdr_peak_luminance_label.Show(show)
+		self.lut3d_hdr_peak_luminance_ctrl.Show(show)
+		self.lut3d_hdr_peak_luminance_ctrl_label.Show(show)
 		self.panel.Layout()
 		self.panel.Thaw()
 	
